//----------S: Include Files ----------
#include <stdlib.h>
#include "LPC177x_8x.h"
#include "lpc177x_8x_clkpwr.h"
#include "lpc177x_8x_pinsel.h"
#include "lpc177x_8x_gpio.h"
#include "stdio.h"
#include "lpc_types.h"
#include "math.h"
#include "lpc177x_8x_uart.h"
#include <rt_sys.h>
#include <File_Config.h>

//----------S: Uart ----------
#define UART0_INTERRUPT					UART0_IRQn
#define UART1_INTERRUPT					UART1_IRQn
#define UART2_INTERRUPT					UART2_IRQn
#define UART3_INTERRUPT					UART3_IRQn
#define UART4_INTERRUPT					UART4_IRQn
#define UART0_IRQHander		UART0_IRQHandler
#define UART1_IRQHander		UART1_IRQHandler
#define UART2_IRQHander		UART2_IRQHandler
#define UART3_IRQHander		UART3_IRQHandler
#define UART4_IRQHander		UART4_IRQHandler
#define UART0_RING_BUFSIZE 1024
#define UART1_RING_BUFSIZE 1024
#define UART2_RING_BUFSIZE 1024
#define UART3_RING_BUFSIZE 1024
#define UART4_RING_BUFSIZE 1024
#define BUF0_MASK (UART0_RING_BUFSIZE-1)
#define BUF1_MASK (UART1_RING_BUFSIZE-1)
#define BUF2_MASK (UART2_RING_BUFSIZE-1)
#define BUF3_MASK (UART3_RING_BUFSIZE-1)
#define BUF4_MASK (UART4_RING_BUFSIZE-1)
#define BUF0_IS_FULL(head, tail) ((tail&BUF0_MASK)==((head+1)&BUF0_MASK))
#define BUF1_IS_FULL(head, tail) ((tail&BUF1_MASK)==((head+1)&BUF1_MASK))
#define BUF2_IS_FULL(head, tail) ((tail&BUF2_MASK)==((head+1)&BUF2_MASK))
#define BUF3_IS_FULL(head, tail) ((tail&BUF3_MASK)==((head+1)&BUF3_MASK))
#define BUF4_IS_FULL(head, tail) ((tail&BUF4_MASK)==((head+1)&BUF4_MASK))
#define BUF0_WILL_FULL(head, tail) ((tail&BUF0_MASK)==((head+2)&BUF0_MASK))
#define BUF1_WILL_FULL(head, tail) ((tail&BUF1_MASK)==((head+2)&BUF1_MASK))
#define BUF2_WILL_FULL(head, tail) ((tail&BUF2_MASK)==((head+2)&BUF2_MASK))
#define BUF3_WILL_FULL(head, tail) ((tail&BUF3_MASK)==((head+2)&BUF3_MASK))
#define BUF4_WILL_FULL(head, tail) ((tail&BUF4_MASK)==((head+2)&BUF4_MASK))
#define BUF0_IS_EMPTY(head, tail) ((head&BUF0_MASK)==(tail&BUF0_MASK))
#define BUF1_IS_EMPTY(head, tail) ((head&BUF1_MASK)==(tail&BUF1_MASK))
#define BUF2_IS_EMPTY(head, tail) ((head&BUF2_MASK)==(tail&BUF2_MASK))
#define BUF3_IS_EMPTY(head, tail) ((head&BUF3_MASK)==(tail&BUF3_MASK))
#define BUF4_IS_EMPTY(head, tail) ((head&BUF4_MASK)==(tail&BUF4_MASK))
#define BUF0_RESET(bufidx)	(bufidx=0)
#define BUF0_INCR(bufidx)	(bufidx=(bufidx+1)&BUF0_MASK)
#define BUF1_RESET(bufidx)	(bufidx=0)
#define BUF1_INCR(bufidx)	(bufidx=(bufidx+1)&BUF1_MASK)
#define BUF2_RESET(bufidx)	(bufidx=0)
#define BUF2_INCR(bufidx)	(bufidx=(bufidx+1)&BUF2_MASK)
#define BUF3_RESET(bufidx)	(bufidx=0)
#define BUF3_INCR(bufidx)	(bufidx=(bufidx+1)&BUF3_MASK)
#define BUF4_RESET(bufidx)	(bufidx=0)
#define BUF4_INCR(bufidx)	(bufidx=(bufidx+1)&BUF4_MASK)
#define No_parity             UART_PARITY_NONE
#define	Odd_parity            UART_PARITY_ODD: 
#define	Even_parity	          UART_PARITY_EVEN 
#define	stick_1_parity        UART_PARITY_SP_1
#define	stick_0_parity        UART_PARITY_SP_0
#define UART_5_bit					  UART_DATABIT_5
#define UART_6_bit						UART_DATABIT_6
#define	UART_7_bit						UART_DATABIT_7
#define UART_8_bit						UART_DATABIT_8
#define	UART_1_Stop_Bits			UART_STOPBIT_1
#define	UART_2_Stop_Bits			UART_STOPBIT_2
#define RING 0
#define TIME_SHARING 1
typedef struct
{
    __IO uint32_t tx_head;                /*!< UART Tx ring buffer head index */
    __IO uint32_t tx_tail;                /*!< UART Tx ring buffer tail index */
    __IO uint32_t rx_head;                /*!< UART Rx ring buffer head index */
    __IO uint32_t rx_tail;                /*!< UART Rx ring buffer tail index */
    __IO uint8_t  tx[UART0_RING_BUFSIZE];  /*!< UART Tx data ring buffer */
    __IO uint8_t  rx[UART0_RING_BUFSIZE];  /*!< UART Rx data ring buffer */
} UART0_RING_BUFFER_T;
typedef struct
{
    __IO uint32_t tx_head;                /*!< UART Tx ring buffer head index */
    __IO uint32_t tx_tail;                /*!< UART Tx ring buffer tail index */
    __IO uint32_t rx_head;                /*!< UART Rx ring buffer head index */
    __IO uint32_t rx_tail;                /*!< UART Rx ring buffer tail index */
    __IO uint8_t  tx[UART1_RING_BUFSIZE];  /*!< UART Tx data ring buffer */
    __IO uint8_t  rx[UART1_RING_BUFSIZE];  /*!< UART Rx data ring buffer */
} UART1_RING_BUFFER_T;
typedef struct
{
    __IO uint32_t tx_head;                /*!< UART Tx ring buffer head index */
    __IO uint32_t tx_tail;                /*!< UART Tx ring buffer tail index */
    __IO uint32_t rx_head;                /*!< UART Rx ring buffer head index */
    __IO uint32_t rx_tail;                /*!< UART Rx ring buffer tail index */
    __IO uint8_t  tx[UART2_RING_BUFSIZE];  /*!< UART Tx data ring buffer */
    __IO uint8_t  rx[UART2_RING_BUFSIZE];  /*!< UART Rx data ring buffer */
} UART2_RING_BUFFER_T;
typedef struct
{
    __IO uint32_t tx_head;                /*!< UART Tx ring buffer head index */
    __IO uint32_t tx_tail;                /*!< UART Tx ring buffer tail index */
    __IO uint32_t rx_head;                /*!< UART Rx ring buffer head index */
    __IO uint32_t rx_tail;                /*!< UART Rx ring buffer tail index */
    __IO uint8_t  tx[UART3_RING_BUFSIZE];  /*!< UART Tx data ring buffer */
    __IO uint8_t  rx[UART3_RING_BUFSIZE];  /*!< UART Rx data ring buffer */
} UART3_RING_BUFFER_T;
typedef struct
{
    __IO uint32_t tx_head;                /*!< UART Tx ring buffer head index */
    __IO uint32_t tx_tail;                /*!< UART Tx ring buffer tail index */
    __IO uint32_t rx_head;                /*!< UART Rx ring buffer head index */
    __IO uint32_t rx_tail;                /*!< UART Rx ring buffer tail index */
    __IO uint8_t  tx[UART4_RING_BUFSIZE];  /*!< UART Tx data ring buffer */
    __IO uint8_t  rx[UART4_RING_BUFSIZE];  /*!< UART Rx data ring buffer */
} UART4_RING_BUFFER_T;

static uint8_t UART0_flag_mode;
extern char serial_buf[300];

void UART0_IRQHander(void);
void UARTX_IntErr(UART_ID_Type UartID,uint8_t bLSErrType);
void UARTX_IntTransmit(UART_ID_Type UartID);
void UARTX_IntReceive(UART_ID_Type UartID);
void UARTX_Receive(UART_ID_Type UartID, uint8_t *rxbuf, uint32_t buflen,uint8_t mode);
uint32_t UARTX_Send(UART_ID_Type UartID,uint8_t *txbuf, uint32_t buflen,uint32_t count,uint8_t mode);
void UARTX_ENABLE(UART_ID_Type UartID,uint32_t Baud_rate,UART_PARITY_Type Parity,UART_DATABIT_Type Databits,UART_STOPBIT_Type Stopbits,uint32_t TRANSMIT_INTERRUPT,uint32_t RECEVE_INTERRUPT,uint8_t set_priority );
void UARTX_send_string(UART_ID_Type UartID,char *string,uint8_t mode);
void UARTX_receive_string(UART_ID_Type UartID,uint8_t *string,uint8_t mode);
void UART1_IRQHander(void);
void UART2_IRQHander(void);
void UART3_IRQHander(void);
void UART4_IRQHander(void);
//----------E: Uart----------
